
#include <stdlib.h>
#include <stdio.h>

#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <getopt.h>
#include <time.h>
#include <signal.h>

#include <sys/socket.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/l2cap.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>

#include "hcid.h"



struct link_key *lk_list = NULL;
struct link_key *tmp;

int lk_list_size = 0;
int lk_list_alloc = 0;



/***********************************************/
void usage(void) {
	fprintf(stderr, 
	"USAGE: lktool [OPTIONS]\n"
	"  -a  bdaddr-linkkey-name\n"
	);
	exit(1);
}




/*****************************************************************************
	remove a linkkey for this BT address
	return 1 if removed, 0 otherwise

*/
int read_lk_list( void )
{

  struct link_key lk;
	int fd, r;

	/* read all the content */
	lk_list_alloc = 8;
	lk_list = malloc(  lk_list_alloc * sizeof( *lk_list ));
	
	fd = open ( HCID_KEY_FILE , O_RDONLY );
	if (fd >= 0) {
		while ((r = read(fd,&lk, sizeof(lk))))
		{
			if (r < sizeof( lk )) {
				//corruption in the linkkey database
				break;
			}
			
			// add this record in lk_list
			if (lk_list_alloc <= lk_list_size+1) { /* we put a +1 to be sure we can add a link key after reading the whole file */
				lk_list_alloc += 8;
				tmp = realloc( lk_list, lk_list_alloc * sizeof( *lk_list ));
				if (tmp == NULL) {
					// no more memory
					break;
				}
				lk_list = tmp;
			}
			memcpy( lk_list + lk_list_size, &lk, sizeof( lk ));
			lk_list_size++;
		}
		close( fd);
	}
	return 0;
}


/******************************************************************************/
int write_lk_list( void )
{
	int fd;
	fd = open( HCID_KEY_FILE, O_WRONLY | O_CREAT | O_TRUNC );
	if (fd >= 0) {
		if (lk_list_size > 0) {
			write( fd, lk_list, lk_list_size * sizeof( *lk_list ));
		}
		close( fd );
	}
	return 0;
}


/******************************************************************************/
const char *lk2str( struct link_key *lk, const char *space)
{
	static char result[100];
	char *ptr = result;
	int i;
	
	for (i=0; i < 15; i++) {
		ptr += sprintf( ptr, "%02X%s", (int)(lk->key[i] & 0xFF), (char *)space );
	}
	sprintf( ptr, "%02X", lk->key[15] & 0xFF );
	return result;
}


/******************************************************************************
* return 0 if OK
*/
int str2lk( const char *lk_str, struct link_key *lk )
{
	char *ptr;
	char tmp[3];
	int i;
	int value;

	ptr = (char *)lk_str;
	for (i=0; i < 16; i++) {
		tmp[0] = ptr[0];
		tmp[1] = ptr[1];
		tmp[2] = 0;
		ptr += 2;
		if (sscanf( tmp, "%x", &value ) == 1) {
			lk->key[i] = value;
		} else {
			return 1;
		}
	}
	return 0;
}






/*****************************************************************************/
void listing( void ) 
{
	bdaddr_t bda;
	int i;

	read_lk_list();
	for (i=0; i < lk_list_size; i++) {
		baswap( &bda, &lk_list[i].dba );
		printf( "%s - %s - %s\n", batostr( &bda ),lk2str( &lk_list[i], "" ) ,lk_list[i].name );
	}
	exit(0);
}


/*****************************************************************************/
void add_link_key( char *bdaddr_str, char *linkkey_str, char *name )
{
	bdaddr_t bda;
	int i;
	struct link_key lk;

	read_lk_list();
	baswap( &bda, strtoba( bdaddr_str ));
	bacpy( &lk.dba, &bda );
	if (str2lk( linkkey_str, &lk )) {
		/* wrong link key */
		fprintf( stderr, "wrong link key\n");
		exit(1);
	}
	strcpy( lk.name, name );
	lk.type = 0;
	memset( &lk.sba, 0, 6 );

	//remove any previous recording
	for (i=0; i< lk_list_size; i++) {
		if (bacmp( &bda, &lk_list[i].dba) == 0) {
			memcpy( &lk_list[i], &lk_list[lk_list_size-1], sizeof( lk_list[i] ));
			lk_list_size--;
		}
	}

	//add this one
	memcpy( &lk_list[lk_list_size++], &lk, sizeof(lk));
	write_lk_list();
	exit(0);
}
	




/*****************************************************************************/
int main(int argc, char **argv) 
{
	int result;
	char bdaddr_str[50];
	char linkkey_str[50];
	char name[255];
	
	while (1) {
		result = getopt( argc, argv, "la:" );
		if (result == -1) break;
		switch (result) 
		{
		case 'l':
			listing();
			break;
			
		case 'a':
			result = sscanf( optarg, "%[0-9a-fA-F:]-%[0-9a-fA-F]-%s", bdaddr_str, linkkey_str, name );
			if (result == 3) {
				add_link_key( bdaddr_str, linkkey_str, name );
			} else {
				usage();
			}
			break;

		}
	} ;
	usage();
	return 1;
}
