#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <fcntl.h>
#include <syslog.h>
#include <errno.h>
#include <time.h>
#include <fcntl.h>
#include <time.h>

#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <asm/types.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>

#include "glib-ectomy.h"

#include "hcid.h"
#include "lib.h"


#define MAX_HCI_DEV 10
#define WATCHOD_POLL_INTERVAL_S  60

#define DEFAULT_WATCHDOG_SCRIPT_PREFIX  "/var/run/hcid/watchdog_hci"

enum {
	WD_DOWN = 0,
	WD_UP,
	WD_REQ_SENT
};


struct {
	int hci_fd;
	int wd_state;
	GIOChannel *chan;
} wd_dev[ MAX_HCI_DEV ];




void watchodg_reload_timer(void) {
	struct itimerval t;

	t.it_value.tv_sec = hcid.watchdog_period;
	t.it_value.tv_usec = 0;
	setitimer( ITIMER_REAL, &t, NULL );

}

void watchdog_unregister( int hcidev )
{
	if ( wd_dev[ hcidev ].wd_state != WD_DOWN ) {
		syslog(LOG_INFO, "watchdog_unregister hci%d", hcidev);
		if (wd_dev[ hcidev ].chan != NULL) {
			g_io_channel_close( wd_dev[ hcidev ].chan );
			wd_dev[ hcidev ].chan = NULL;
		}
		close( wd_dev[ hcidev ].hci_fd );
		wd_dev[ hcidev ].hci_fd = -1;
		wd_dev[ hcidev ].wd_state = WD_DOWN;
	} else {
		syslog(LOG_INFO, "watchdog_unregister hci%d but not registered yet", hcidev);
	}
}


void watchodg_timer_signal(int num)
{
	int i;
	char script[64];
	struct stat dummy;
	
	syslog(LOG_INFO, "watchodg_timer_signal");
	
	for (i=0; i < MAX_HCI_DEV; i++) {
		syslog(LOG_INFO, "  hci%d,  state %d,  fd %d", i, wd_dev[i].wd_state, wd_dev[i].hci_fd );
		switch (wd_dev[i].wd_state) {
		case WD_UP:
			/* send a request, only if the watchdog script is here */
			sprintf( script, "%s%d", hcid.watchdog_script_prefix ? hcid.watchdog_script_prefix : DEFAULT_WATCHDOG_SCRIPT_PREFIX, i );
			if (stat( script, &dummy ) == 0) {
				hci_send_cmd( wd_dev[i].hci_fd, OGF_INFO_PARAM, OCF_READ_LOCAL_VERSION, 0, NULL );
				wd_dev[i].wd_state = WD_REQ_SENT;
			} else {
				syslog(LOG_INFO, "   script %s not here", script );
			}
			break;
			
		case WD_REQ_SENT:
			/* we didn't catch the answer */
			watchdog_unregister( i );
			
			syslog(LOG_INFO, "hcid watchdog on hci%d.", i);
			
			sprintf( script, "%s%d watchdog hci%d", hcid.watchdog_script_prefix ? hcid.watchdog_script_prefix : DEFAULT_WATCHDOG_SCRIPT_PREFIX, i, i );
			system( script );
			break;
		}
	}
	watchodg_reload_timer();
}
			

void watchdog_start(void)
{
	printf("watchdog_start  enable %d,  path %s", hcid.watchdog, hcid.watchdog_script_prefix);
	memset( &wd_dev, 0, sizeof( wd_dev) );
	
	if (hcid.watchdog_period == 0) hcid.watchdog_period = WATCHOD_POLL_INTERVAL_S;
	signal( SIGALRM, watchodg_timer_signal );
	watchodg_reload_timer();
}

gboolean io_watchdog_event(GIOChannel *chan, GIOCondition cond, gpointer data)
{
	char buf[HCI_MAX_EVENT_SIZE], *ptr = buf;
	int hcidev = (int) data;
	int len, type, dev;
	hci_event_hdr *eh;
	GIOError err;
	
	syslog(LOG_INFO, "io_watchdog_event");

	if (cond & (G_IO_NVAL | G_IO_HUP | G_IO_ERR)) {
		if (wd_dev[ hcidev ].chan != NULL) {
			g_io_channel_close(chan);
			//free(data);
			wd_dev[ hcidev ].chan = NULL;
		}
		watchdog_unregister( hcidev );
		return FALSE;
	}
	
	if ((err = g_io_channel_read(chan, buf, sizeof(buf), &len))) {
		if (err == G_IO_ERROR_AGAIN)
			return TRUE;
		if (wd_dev[ hcidev ].chan != NULL) {
			g_io_channel_close(chan);
			//free(data);
			wd_dev[ hcidev ].chan = NULL;
		}
		watchdog_unregister( hcidev );
		return FALSE;
	}

	type = *ptr++;

	if (type != HCI_EVENT_PKT)
		return TRUE;

	
	/*
	eh = (hci_event_hdr *) ptr;
	ptr += HCI_EVENT_HDR_SIZE;


	switch (eh->evt) {
	case EVT_PIN_CODE_REQ:
		pin_code_request(dev, &di->bdaddr, (bdaddr_t *) ptr);
		break;

	case EVT_LINK_KEY_REQ:
		link_key_request(dev, &di->bdaddr, (bdaddr_t *) ptr);
		break;

	case EVT_LINK_KEY_NOTIFY:
		link_key_notify(dev, &di->bdaddr, ptr);
		break;

	//AMO{begin : on enregistre le nom }
	case EVT_REMOTE_NAME_REQ_COMPLETE:
		name_notify(dev, &di->bdaddr, ptr);
		break;
	//AMO{end}
	}
	*/
	
	/* if we receive at least a command complete event, it means that the device is still here */
	
	switch (wd_dev[hcidev].wd_state) {
	case WD_REQ_SENT:
		wd_dev[hcidev].wd_state = WD_UP;
		break;
	}
		
	return TRUE;
}

void watchdog_register( int hcidev )
{
	int s;
	struct hci_filter flt;
	
	syslog(LOG_INFO, "watchdog_register hci%d", hcidev);
	
	if (hcidev >= MAX_HCI_DEV) {
		syslog(LOG_ERR, "watchdog: hcidev number too high (%d)", hcidev);
		return;
	}
	
	watchdog_unregister( hcidev );
	if ((s = hci_open_dev(hcidev)) < 0) {
		syslog(LOG_ERR, "Can't open device hci%d. %s(%d)", hcidev, strerror(errno), errno);
		return;
	}
	/* Set filter */
	hci_filter_clear(&flt);
	hci_filter_set_ptype(HCI_EVENT_PKT, &flt);
	hci_filter_set_event(EVT_CMD_COMPLETE, &flt);
	if (setsockopt(s, SOL_HCI, HCI_FILTER, &flt, sizeof(flt)) < 0) {
		syslog(LOG_ERR, "Can't set filter on hci%d. %s(%d)", 
				hcidev, strerror(errno), errno);
		close(s);
		return;
	}
	/* set the socket in non blocking */	
	fcntl( s, F_SETFL, O_NONBLOCK );
	
	/* record the info */
	wd_dev[ hcidev ].hci_fd = s;
	wd_dev[ hcidev ].wd_state = WD_UP;
	
	/* link it with event loop */
	wd_dev[ hcidev ].chan = g_io_channel_unix_new( s );
	g_io_add_watch( wd_dev[ hcidev ].chan , G_IO_IN | G_IO_NVAL | G_IO_HUP | G_IO_ERR,
			io_watchdog_event, (void *) hcidev);
}


