/*
 * Set affinity and real-time policy of a process
 *
 * Copyright 2006 Vivien Chappelier -- Inventel/Thomson
 *   Inspired from renice
 *
 * Code Modified by Thomson. Revision 1.0.0
 */

#include <sched.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include "busybox.h"

/* Hmm.. that syscall doesn't seem implemented in our libc :-( */
#include <sys/syscall.h>
#include <unistd.h>
int sched_setaffinity(pid_t pid, size_t cpusetsize, unsigned long *cpuset)
{
	return syscall(__NR_sched_setaffinity, pid, cpusetsize, cpuset);
}

int rttune_main(int argc, char **argv)
{
	struct sched_param schp;
	pid_t pid;
	int prio;
	unsigned long mask;
	char *pol;
	char *aff;
	int policy;

	if (argc < 4)	bb_show_usage();

	pid = strtol(argv[1], NULL, 10);
	prio = strtol(argv[2], &pol, 10);
	if (prio > 99)		prio = 99;
	if (prio <  0)		prio =  0;
	aff = argv[3];
	mask = 0;
	do
		mask |= 1 << strtol(aff, &aff, 10);
	while(*aff == '+');

	policy = SCHED_OTHER;

	if (*pol == 'R') {
		policy = SCHED_RR;
		pol++;
	} else if(*pol == 'F') {
		policy = SCHED_FIFO;
		pol++;
	} else
		policy = SCHED_OTHER;

	if (prio == 0 && policy != SCHED_OTHER) {
		bb_perror_msg("zero static priority but"
			      "real-time policy selected");
		return EXIT_FAILURE;
	}
	if (prio > 0 && policy == SCHED_OTHER) {
		bb_perror_msg("non-zero static priority but no"
			      "real-time policy selected");
		return EXIT_FAILURE;
	}

	schp.sched_priority = prio;

	if (sched_setscheduler(pid, policy, &schp)) {
			bb_perror_msg("%d: sched_setscheduler", pid);
			return EXIT_FAILURE;
	}
	
	if (sched_setaffinity(pid, sizeof(mask), &mask)) {
			bb_perror_msg("%d: sched_setaffinity", pid);
			return EXIT_FAILURE;
	}

	return EXIT_SUCCESS;
}
