/**
 * @file   get_param.c
 * @author Clement Moreau <clement.moreau@inventel.fr>, Xavier Debreuil <xavier.debreuil@inventel.fr>
 * @date   Sat Mar 29 17:09:22 2003
 * 
 * @brief  r�cup�ration des param�tres d'usine dans le psi/nvram ou dans le block RedBoot config
 * 
 *
 * Code Modified by Thomson. Revision 1.0.0
 * 
 */

#include <stdio.h>
//#include "hard_api/board_api.h"
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>

#include <hco_rip_api.h>

//#define DEBUG
/* for debugging purposes only */
#ifdef DEBUG
#define diag_printf printf
#else
#define diag_printf(fmt,args...)
#endif

// displayed as hex, while it is human-readable ascii. can not
// change it because existing scripts rely on it :-(
unsigned char mac_base_str[18] = "FF:FF:FF:FF:FF:FF";
unsigned char wep_key_str[11] = "FFFFFFFFFF";
unsigned char bl_version_str[9] = "FFFFFFFF";
//unsigned char serial_number_str[17] = "0000000000000000";
//unsigned char inventel_wireless_magic_str[256] = "0xFFFFFFFF";
//unsigned char h235_key_str[256] = "1234567812345678";
unsigned char boot_loader_build_id[] = "CFE";
unsigned char bt_pin[5] = "0000";

unsigned char buffer[2048];// max size of any rip entry
int id = 0;
unsigned long length = 0;

int noOption = 0;
int showId = 0;
int showMacAddress = 0;
int showWepKey = 0;
int showBtPin = 0;
int showBlVersion = 0;

void get_param_exit(void){
	int i;
	if (noOption) {
		printf("MAC_BASE=%s\nWEP_KEY=%s\nBOOT_LOADER_BUILD_ID=\"%s\"\n",
			mac_base_str, wep_key_str, boot_loader_build_id);
	} else if (showMacAddress) {
		printf("MAC_BASE=%s\n", mac_base_str);
	} else if (showWepKey) {
		printf("WEP_KEY=%s\n", wep_key_str);
	} else if (showBtPin) {
		printf("BT_PIN=%s\n", bt_pin);
	} else if (showBlVersion) {
		printf("BL_VERSION=%s\n", bl_version_str);
	} else if (showId) {
		printf("field %d: ", id);
		if(length == 0){
		  printf("value not available\n");
		}else{
		  for(i=0;i<length;i++){
            if(i%20 == 0){
      	      fprintf(stdout, "\n  ");// spread long output over multiple lines
            }
            printf("%02x ", (unsigned char)buffer[i]);
		  }
		  printf("\n");
		}
  	}
    exit(0);
}

static void help(){
  printf("\n");
  printf("Usage: get_param [-d id | -m | -w | -p | -v | -h]\n");
  printf("Options are as follows:\n");
  printf(" -d id   obtain field value by its id\n");
  printf(" -m      show mac address\n");
  printf(" -w      show wep key\n");
  printf(" -p      show bluetooth pin\n");
  printf(" -v      show bootloader version\n");
  printf("\n");
}

int get_param_main(int argc, char **argv){
  int opt;
  
  if (argc > 1){
    if ((opt = getopt(argc, argv,"d:mwspvh")) != -1){
      switch (opt){
      case 'd':
        showId = 1;
        id = (unsigned short)strtoul(optarg, NULL, 10);
        break;
      case 'm':
        showMacAddress = 1;
        break;
      case 'w':
        showWepKey = 1;
        break;
      case 'p':
        showBtPin = 1;
        break;
      case 'v':
        showBlVersion = 1;
        break;
      case 'h':
      case '?':
      default:
        help();
        return 0;
      }
    }
  }else{
  	noOption = 1;
    showMacAddress = 1;
    showWepKey = 1;
  }
 
  if(showMacAddress){
    // get mac address
    id = RIP_COMMON_WAN_ADDR;
    length = RIP_PRIV_WAN_ADDR_SIZE;
    if(hco_rip_read(id, buffer, &length)<0){
  	  // default value will be used
    }else{
      sprintf(mac_base_str, "%02x:%02x:%02x:%02x:%02x:%02x", buffer[0],
	  buffer[1], buffer[2], buffer[3], buffer[4], buffer[5]);
    }
  }
  
  if(showWepKey){
    // get wep key
    if(getenv("_WEPKEY_SERIAL")!= NULL){
    	strncpy(wep_key_str, getenv("_WEPKEY_SERIAL"),10);
    }
  }
   
  if(showBlVersion){
    // get bootloader version
    id = RIP_PRIV_BOOTLOADER_VERSION;
    length = RIP_PRIV_BOOTLOADER_VERSION_SIZE;
    if(hco_rip_read(id, buffer, &length)<0){
  	  // default value will be used
    }else{
      sprintf(bl_version_str, "%02x%02x%02x%02x", buffer[0],
	    buffer[1], buffer[2], buffer[3]);
    }
  }
    
  if(showId){
  	length = 2048;
    if(hco_rip_read(id, buffer, &length)<0){
      length = 0; // to indicate that there is no value
    }else{
      // ok 
    }  	
  }
  
  get_param_exit();

  return 0;
}
