/***************************************************************************
 * File Name  : ledctrl.c
 *
 * Description: 
 *
 *    This application control the led action by calling sysLedCtrl.
 *    smaple calls:     sysLedCtrl(kLedAdsl, kLedStateBlinkOnce);           // kLedxxx defines in board_api.h
 *
 * Created on :  11/01/2002  seanl
 *
 ***************************************************************************/
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>  /* ioctl */

#include "busybox.h"
#include "hard_api/board_api.h"

typedef struct
{
    char *validLedName;
    BOARD_LED_NAME ledName;
} LED_NAME_MAP, *PLED_NAME_MAP;


// LED name as the first param
const LED_NAME_MAP ledNameMap[] =
{
    {"Adsl",         kLedAdsl},
    {"Wireless",     kLedWireless},
    {"Voip",         kLedVoip},
    {"SelfTest",     kLedSelfTest},
    {"Lan",          kLedLan},
    {"Led1",	       kLed1},
    {"Led2",	       kLed2},
    {"Led3",	       kLed3},
    {"Led4",	       kLed4},
    {"Led5",	       kLed5},
    /*added by songxq@12-22*/
    {"Power",        kLedPower},
    {"InternetRed",  kLedInternetRed},
    {"InternetGreen",kLedInternetGreen},
    {"Update",       kLedUpgrade},
    {"LanAct",       kLedEmac1Act},
    {"LanSpeed",     kLedEmac1Speed},
    /*end of added by songxq@12-22*/
		{"All",	     kLed1+kLed2+kLed3+kLed4+kLed5},
    {NULL,           kLedEnd},
};

typedef struct 
{
    char *validLedState;
    BOARD_LED_STATE ledState;
} LED_STATE_MAP, *PLED_STATE_MAP;

const LED_STATE_MAP ledStateMap[] =
{
    {"Off",                 kLedStateOff},               
    {"On",                  kLedStateOn},
    {"BlinkOnce",           kLedStateBlinkOnce},         
    {"SlowBlinkContinues",  kLedStateSlowBlinkContinues},
    {"FastBlinkContinues",  kLedStateFastBlinkContinues},
    {"BlinkContinues",	     6},
    {NULL,                  -1},
};

static void showUsage()
{
    PLED_NAME_MAP namePtr = (PLED_NAME_MAP)(&ledNameMap);
    PLED_STATE_MAP statePtr = (PLED_STATE_MAP)(&ledStateMap);

    printf("\nUsage: ledctrl led_name led_state\n\n");
    printf("Supported led names:\n");
    while (namePtr->validLedName)
        printf("%s\n", namePtr++->validLedName);
    
    printf("\nSupported led states:\n");
    while (statePtr->validLedState)
        printf("%s\n", statePtr++->validLedState);

    exit(-1);
}

int ledctrl_main(int argc, char **argv)
{
    PLED_NAME_MAP namePtr = (PLED_NAME_MAP)&ledNameMap;
    PLED_STATE_MAP statePtr = (PLED_STATE_MAP)&ledStateMap;

    if (argc < 3)
        showUsage();

    while (namePtr->validLedName)
        if (strcmp(namePtr->validLedName, argv[1]) == 0)
            break;
        else
            namePtr++;
    if (!namePtr->validLedName)
        showUsage();

    while (statePtr->validLedState)
        if (strcmp(statePtr->validLedState, argv[2]) == 0)
            break;
        else
            statePtr++;

    if (!statePtr->validLedState)
        showUsage();


    if (strcmp(argv[1], "All") == 0)
    {
    	//for production test
	printf("Same state for all LED\n");
	sysLedCtrl(kLed1, statePtr->ledState);
	sysLedCtrl(kLed2, statePtr->ledState);
	sysLedCtrl(kLed3, statePtr->ledState);
	sysLedCtrl(kLed4, statePtr->ledState);
	sysLedCtrl(kLed5, statePtr->ledState);

	//Backlight led
	if (kLedStateOn == statePtr->ledState)
		system("hcitool cmd 0x3A 0x200 ff 00 01 00\n");
	else if (kLedStateOff == statePtr->ledState)
		system("hcitool cmd 0x3A 0x200 00 00 01 00\n");
    }
    else {
	if (strcmp(argv[2], "BlinkContinues") == 0)
	{
		int on, off;

		if (argc < 5) {
			on = 10;
			off = 5;
		} else {
			on = atoi(argv[3]);
			off = atoi(argv[4]);
		}
		sysLedCtrlBlink(namePtr->ledName, on, off);
	} else
		sysLedCtrl(namePtr->ledName, statePtr->ledState);
    }

    return 0;

}

