#include "Lzmalib.h"
#include "unseal.h"

static int Lzma_fill_in_properties(lz_stream *lz)
{   
     int i;
     unsigned char b;
     unsigned prop0;
     
     lz->outSize = 0;
     if(!lz->bo.lzread((t_envelope *)lz->bo.fpe,lz->properties,(sizeof(unsigned char)*5)))
        return 0;

     for(i = 0 ; i<4; i++)
     {
       if(!lz->bo.lzread((t_envelope *)lz->bo.fpe,&b,sizeof(b)))
          return 0;
       lz->outSize +=(unsigned int)(b) <<(i*8);
     }

     if(lz->outSize == 0xFFFFFFFF)
        return 0;

     for(i = 0 ; i<4 ; i ++)
     {
       if(!lz->bo.lzread((t_envelope *)lz->bo.fpe,&b,sizeof(b)))
	 return 0;
       if(b!=0)
	 return 0;
     }
     prop0 = lz->properties[0];
     if (prop0 >= (9*5*5))
     {
         return 0;
     }
     for (lz->pb = 0; prop0 >= (9 * 5); lz->pb++, prop0 -= (9 * 5));
     for (lz->lp = 0; prop0 >= 9;lz->lp++, prop0 -= 9);
          lz->lc = prop0;
     
     lz->lzmaInternalSize = ((LZMA_BASE_SIZE + (LZMA_LIT_SIZE << (lz->lc+lz->lp))) * sizeof(CProb))+100;
     lz->lzmaInternalData = lz->malloc(lz->lzmaInternalSize);
     if(lz->lzmaInternalData == 0)
        return(0);
     for (i = 0; i < 4; i++)
      lz->dictionarySize += (UInt32)(lz->properties[1 + i]) << (i * 8);

     lz->dictionary = lz->malloc(lz->dictionarySize);
     if(lz->dictionary == 0)
     {
        return 0;
     }
	  
     return 1; 
     
 }

int LzmaInit(lz_stream *lz)
{
  int res = 1;

  res = Lzma_fill_in_properties(lz);
  LzmaDecoderInit(lz->lzmaInternalData, lz->lzmaInternalSize, lz->lc, lz->lp, lz->pb, lz->dictionary, lz->dictionarySize,&lz->bo.InCallback);

  return res;
}

int LzmaReadCompressed(void *object, unsigned char **buffer, unsigned int *size)
{
   CBuffer *bo = (CBuffer *)object;
   int len=0;
   *size = bo->Size; /* You can specify any available size here */
   len = bo->lzread((t_envelope *)bo->fpe, bo->Buffer, bo->Size);
   if(len < bo->Size)
     {
       *size = len;
     }  
   *buffer = bo->Buffer;
   return LZMA_RESULT_OK;
}

int LzmaDecompress (lz_stream *lz)
{
  int BlockSize = lz->bo.Size;
  int res = 0;
  int latestblock = 0;
  int read_block = 256;
  static long read = 0;
  
  if((read >= lz->outSize-lz->bo.Size) && !latestblock)
    {
      BlockSize = lz->outSize - read;
      latestblock = 1;
    }      

  res = LzmaDecode(lz->lzmaInternalData,lz->outStream, BlockSize, &lz->outSizeProcessed);
  
  read += lz->outSizeProcessed;
  lz->avail_out = read_block-lz->outSizeProcessed;

  if(res != 0)
    {
      return LZMA_DECODE_ERROR;
    }

  if(latestblock)
    {      
      return LZMA_STREAM_END;
    }

  return LZMA_OK;
}  

void LzmaDecompressEnd(lz_stream *lz)
{
  lz->free(lz->dictionary);
  lz->free(lz->lzmaInternalData);
}
