/**
 * @file   fix.c
 * @author Clement Moreau <clement.moreau@inventel.fr>
 * @date   Mon Mar 21 17:56:18 2005
 * 
 * @brief  fix les diffrents paramtres qui doivent tre fixs pour afone
 *         
 * Copyright INVENTEL -- 2004-2005
 * 
 * 
 */


#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <conffile/conffile.h>

#if 0
#define DIAG_PRINTF printf
#else
#define DIAG_PRINTF(...)
#endif
#define STR_LENGTH 1000

int main (int argc,char* argv[] ) {
  FILE * pfile;
  FILE * fptr;
  int gwname_only=0;
  
  char proc_mtd_str[STR_LENGTH],  redboot_config_mtd_str[STR_LENGTH],    system_str[STR_LENGTH], * ptr0, *ptr;
  char mtd[STR_LENGTH] ; 
  
  if (argc>1) {
	  fprintf(stderr, "Argv: %s\n", argv[1]);
	  if (!strcmp(argv[1], "-gw")) 
	  {
		  fprintf(stderr, "Update gwname only\n");
		  gwname_only=1;
	  }
  }
  
  fptr = fopen("/proc/mtd", "r");
  if(!fptr) {
    printf("Unable to open /proc/mtd\n");
    return 127 ; 
  }
  else {
    fread((void *)proc_mtd_str, sizeof(unsigned char), STR_LENGTH, fptr);
    DIAG_PRINTF("%s\n", proc_mtd_str);
  }
  fclose(fptr);
  
  
  // look for the number of RedBoot config mtd partition
  ptr = strstr(proc_mtd_str, "RedBoot config");
  
  // RedBoot config partition is never the first partition :
  // there is at least one partition (RedBoot) before
  do {
    ptr --;
  } while (*ptr != '\n');
  ptr ++;
  // ptr points to : mtd7: 00001000 00010000 "RedBoot config"
  
  ptr0 = redboot_config_mtd_str;
  while (*ptr != ':')
    *ptr0 ++ = *ptr ++;
  *ptr0 = 0;
  
  DIAG_PRINTF("MTD RedBoot config partition : %s\n", redboot_config_mtd_str);
  
  sprintf(system_str,"redboot_fconfig_get --parameter af_adsl_pass /dev/%s",redboot_config_mtd_str);
  strcpy (mtd, redboot_config_mtd_str);
  DIAG_PRINTF("cmd : %s\n",system_str);
  pfile = popen (system_str,"r");
  
  if (!pfile) {
    DIAG_PRINTF("pfile nul \n");
    return 127;
  }
  
  if (!fgets(ptr, STR_LENGTH, pfile)) {
    DIAG_PRINTF("fgets nul\n");
    return 127;
  }
  
  ptr0=strchr(ptr,'\n');
  if (ptr0) {
    *ptr0=0;
    DIAG_PRINTF("Ici\n");
  }

  DIAG_PRINTF("--%s--\n",ptr);
  if ( gwname_only != 1) {
  conf_set_str("/etc/adsld.conf" , "ADSL_PPP_PASS", ptr,STR_LENGTH-1);
  conf_set_str("/etc/firm.conf" , "UPDATE_PASSWORD", ptr,STR_LENGTH-1);
  
  conf_get_str("/etc/hardware.conf", "ETHER_0", ptr, STR_LENGTH-16); /* rservation 16 octets pour le strcat 
								      * qui suit */
  ptr0 = ptr + 19 ; 
  ptr0[-1]=0;
  int j,i =0 ,k=0; 
  for (j=0;j<6;j++) {
    ptr0[k++]=ptr[i++];
    ptr0[k++]=ptr[i++];
    i++;
  }
  ptr0[i]=0;

  /*update de firm.conf */
  conf_set_str("/etc/firm.conf" , "UPDATE_USERNAME", ptr0, STR_LENGTH-1);
  
  strcat (ptr0,"@adsl.afone.com");
  DIAG_PRINTF("--%s--\n",ptr0);
  conf_set_str("/etc/adsld.conf", "ADSL_PPP_NAME", ptr0, STR_LENGTH-1);
  
  /* remove chap and pap files */
  remove ("/etc/ppp/chap-secrets");
  remove ("/etc/ppp/pap-secrets");

  pclose (pfile);
  }
  /* GW NAME */

  sprintf(system_str,"redboot_fconfig_get --parameter af_gwname /dev/%s",mtd);
  DIAG_PRINTF("cmd : %s\n",system_str);
  pfile = popen (system_str,"r");
  
  if (!pfile) {
    DIAG_PRINTF("pfile nul \n");
    return 127;
  }
  
  if (!fgets(ptr, STR_LENGTH, pfile)) {
    DIAG_PRINTF("fgets nul\n");
    return 127;
  }
  
  ptr0=strchr(ptr,'\n');
  if (ptr0) {
    *ptr0=0;
    DIAG_PRINTF("Ici\n");
  }

  DIAG_PRINTF("--%s--\n",ptr);

  pclose (pfile);

  conf_set_str("/etc/mgcp-voip.conf" , "gwName", ptr,STR_LENGTH-1);

  return 0;
}
